/*
 * @(#)Burn.java
 *
 * This file is part of webCDwriter - Network CD/DVD Writing.
 *
 * Copyright (C) 1999-2005 Jrg P. M. Haeger
 *
 * webCDwriter is free software. See CDcreator.java for details.
 */

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.io.*;
import java.text.*;
import javax.swing.*;

/**
 * Abstract parent of the burn command classes.
 *
 * @version 20050831
 * @author Jrg P. M. Haeger
 */
abstract public class Burn extends Command {
	final static String
	hintBlankingTOC = "blanking TOC",
	hintCDdetected = "CD detected",
	hintCDrecordStarted = "cdrecord started",
	hintCDwasClosed = "CD was closed",
	hintDVDFormatStarted = "dvd-format started",
	hintFixating = "fixating",
	hintGrowisofsStarted = "growisofs started",
	hintImageReady = "image ready",
	hintMakingImage = "making image",
	hintPercentDone = "percentDone",
	hintVerifying = "verifying",
	hintWriterReserved = "writer reserved"
	;

	final int
		INIT = 0,
		WAITING_FOR_WRITER = 1,
		MEDIUM_DETECTED = 2,
		VERIFY = 3;
	int mode = INIT;

	protected JOptionPane optionPane = null, subPane = null;
	BurnDialog burnDialog;
	JPanel writersPanel;
	int totalKB, oldFreeKB, newFreeKB;
	boolean CDwasClosed = false;
	final String backButton = i18n("MainWin.back");
	final JButton offlineButton = new JButton(i18n("burnOffline"));
	Object[] options = { i18n("details"), okButtonText };
	String burnSessionResult;
	boolean burnSessionResultOK;

	Burn() {
		burnDialog = new BurnDialog(this);
		writersPanel = new JPanel();
		writersPanel.setLayout(
			new BoxLayout(writersPanel, BoxLayout.Y_AXIS));
		writersPanel.setAlignmentX(Box.LEFT_ALIGNMENT);
		offlineButton.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					String[] options = { backButton, okButtonText };
					int res = showOptionDialog(
						i18n("willNowExit"),
						i18n("burnOffline"),
						JOptionPane.INFORMATION_MESSAGE,
						JOptionPane.DEFAULT_OPTION,
						null, options, backButton);
					if (res == 1) {
						writeHint("setBurnOffline");
						try {
							CDcreator.outStream.close();
						}
						catch (IOException e2) {
						}
						offline = true;
						closeDialog();
					}
				}
			});
	}

	void cancelCommand() {
		Version.debug("cancel", "button clicked");

		super.cancelCommand();

		if (mode == VERIFY)
			writeHint("cancel");
		else
			writeCommandIgnoreException("noop");
	}

	private JDialog createErrorDialog() {
		JPanel vbox = new JPanel();
		vbox.setLayout(new BoxLayout(vbox, BoxLayout.Y_AXIS));
		if (burnSessionResult.indexOf("http://") >= 0)
			vbox.add(MainWin.createViewer(lastResult));
		else {
		vbox.add(new JLabel(burnSessionResult));
		if (burnSessionResult.indexOf(
				"power calibration area error") >= 0) {
			JComponent viewer = CDcreator.getHTML("errors/037303.html");
			if (viewer != null && !CDcreator.isNetscapeVM()) {
				vbox.add(Box.createVerticalStrut(10));
				JScrollPane scroller = new JScrollPane();
				scroller.setPreferredSize(new Dimension(440, 200));
				scroller.setAlignmentX(Box.LEFT_ALIGNMENT);
				scroller.getViewport().add(viewer);
				vbox.add(scroller);
			}
		}
		}
		optionPane = new JOptionPane(
			vbox,
			JOptionPane.ERROR_MESSAGE,
			JOptionPane.DEFAULT_OPTION,
			null, options);
		return optionPane.createDialog(
			CDcreator.mainWin, i18n("error"));
	}

	protected JDialog createSuccessDialog() {
		MessageFormat formatter = new MessageFormat("");
		formatter.setLocale(CDcreator.locale);
		if (CDwasClosed)
			formatter.applyPattern(
				i18n("burnSuccessCDwasClosed"));
		else
			formatter.applyPattern(i18n("burnSuccess"));

		int writtenKB = oldFreeKB - newFreeKB;
		if (totalKB < 1024 * 1024) {
			// is a CD-R(W)
			writtenKB -= 2 * (4500 + 2250);
			if (oldFreeKB == totalKB)
				// lead out of the first session
				writtenKB -= 2 * 4500;
			if (writtenKB < 0)
				writtenKB = 0;
		}

		Object[] args = {
			new Integer(writtenKB / 1024),
			new Integer(newFreeKB / 1024),
			new Integer(totalKB / 1024) };

		optionPane = new JOptionPane(
			formatter.format(args),
			JOptionPane.INFORMATION_MESSAGE,
			JOptionPane.DEFAULT_OPTION,
			null, options);
		return optionPane.createDialog(
			CDcreator.mainWin, i18n("titleBurnResult"));
	}

	abstract protected void execBurnCommand();

	protected void execOptions() {
	}

	protected void execSetProjectInfo() {
		if (!CDcreator.askForProjectInfo)
			return;

		String from = Options.projectInfo.from;
		if (from.length() == 0)
			from = "-";
		String to = Options.projectInfo.to;
		if (to.length() == 0)
			to = "-";
		String label = Options.projectInfo.label;
		if (label.length() == 0)
			label = "-";
		String contents = Options.projectInfo.contents;
		if (contents.length() == 0)
			contents = "-";
		execCommand("setProjectInfo "
			+ spaceToEsc(from) + " " + spaceToEsc(to)
			+ " " + spaceToEsc(label) + " " + spaceToEsc(contents));
	}

	Dimension getPreferredSize() {
		return new Dimension(540, 280);
	}

	protected void handleWindowEvent(WindowEvent e) {
		Version.debug("show", "handleWindowEvent(" + e + ")");
		if (mode == WAITING_FOR_WRITER || mode == VERIFY)
			cancelCommand();
	}

	void parseCapacityHint(String lineStr) throws IOException {
		StreamTokenizer tokenizer
			= new StreamTokenizer(
				new StringReader(lineStr));

		tokenizer.nextToken();
		if (!tokenizer.sval.equals("capacity"))
			return;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		totalKB = (int)tokenizer.nval;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		oldFreeKB = (int)tokenizer.nval;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		newFreeKB = (int)tokenizer.nval;
	}

	void parseCDrecordLine(String lineStr) throws IOException {
		StreamTokenizer tokenizer
			= new StreamTokenizer(
				new StringReader(lineStr));
		tokenizer.wordChars(':', ':');

		tokenizer.nextToken();
		if (!tokenizer.sval.equals("Track"))
			return;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		int track = (int)tokenizer.nval;

		if (tokenizer.nextToken() != StreamTokenizer.TT_WORD)
			return;
		if (!tokenizer.sval.equals(":"))
			return;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		int n = (int)tokenizer.nval;

		if (tokenizer.nextToken() != StreamTokenizer.TT_WORD)
			return;
		if (!tokenizer.sval.equals("of"))
			return;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		int N = (int)tokenizer.nval;

		burnDialog.setRange(0, N);
		burnDialog.setProgress(n);

		if (oldFreeKB == 0)
			oldFreeKB = 1024 * N;
	}

	void parsePercentDoneHint(String lineStr) throws IOException {
		StreamTokenizer tokenizer
			= new StreamTokenizer(
				new StringReader(lineStr));

		tokenizer.nextToken();
		if (!tokenizer.sval.equals(hintPercentDone))
			return;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		burnDialog.setProgress((int)tokenizer.nval);
	}

	void parseRankHint(String lineStr) throws IOException {
		StreamTokenizer tokenizer
			= new StreamTokenizer(
				new StringReader(lineStr));

		tokenizer.nextToken();
		if (!tokenizer.sval.equals("rank"))
			return;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		int rank = (int)tokenizer.nval;

		JPanel vp = new JPanel();
		vp.setLayout(new BoxLayout(vp, BoxLayout.Y_AXIS));

		JPanel hp = new JPanel();
		hp.setBorder(BorderFactory.createEmptyBorder(5, 0, 5, 0));
		hp.setLayout(new BoxLayout(hp, BoxLayout.X_AXIS));
		hp.add(new JLabel(i18n("queue") + " " + rank));
		hp.add(Box.createHorizontalStrut(10));

		JButton button = new JButton(i18n("showStatus"));
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				CDcreator.showServerDocument("/status.html");
			}
		});
		hp.add(button);
		hp.setAlignmentX(Box.LEFT_ALIGNMENT);
		vp.add(hp);

		QueueView qv = new QueueView(rank);
		qv.setAlignmentX(Box.LEFT_ALIGNMENT);
		vp.add(qv);
		vp.add(writersPanel);

		optionPane.setMessage(vp);
		validate();
	}

	protected void process(int code, String text) {
//		System.out.println("code = " + code + ", text = " + text);

		if (text.indexOf("eject") >= 0)
			Log.put(text);

		String no = "";
		if (writer >= 0)
			no = "W" + writer + " ";

		switch (code) {
			case 199:
				Version.debug("state", text);
				String a = text.substring(0, 2);
				java.awt.Component[] cs = writersPanel.getComponents();
				int i = 0;
				for (; i < cs.length; i++) {
					JLabel l = (JLabel) cs[i];
					String b = l.getText().substring(0, 2);
					if (b.equals(a)) {
						l.setText(text);
						break;
					}
				}
				if (i == cs.length)
					writersPanel.add(new JLabel(text));
//				activeDialog.pack();
				break;
			case 900:
				String str = i18n("MainWin.unexpectedNonEmptyCD");
				if (showConfirmDialog(str,
						i18n("warning"), JOptionPane.YES_NO_OPTION) == 0)
					writeHint(no + "setAcceptNonEmptyCD");
				break;
			case 901:
				if (text.indexOf("eject") >= 0)
					break;
				str = i18n("MainWin.unexpectedMedium");
				str = CDcreator.replaceAll(str, "@MEDIUM@", text);
				if (showConfirmDialog(str,
						i18n("warning"), JOptionPane.YES_NO_OPTION) == 0)
					writeHint(no + "setAcceptMedium");
				break;
			case 902:
				str = i18n("MainWin.unexpectedEmptyCD");
				if (showConfirmDialog(str,
						i18n("warning"), JOptionPane.YES_NO_OPTION) == 0)
					writeHint(no + "setAcceptEmptyCD");
				break;
			case 903:
				// medium has user ID
				break;
			case 904:
				str = i18n("MainWin.unexpectedWithoutUserID");
				if (showConfirmDialog(str,
						i18n("warning"), JOptionPane.YES_NO_OPTION) == 0)
					writeHint(no + "setAcceptWithoutUserID");
				break;
			case 905:
				String userID = "unknown";
				int p0 = text.indexOf("\"");
				if (p0 >= 0) {
					int p1 = text.indexOf("\"", p0 + 1);
					if (p1 > p0)
						userID = text.substring(p0 + 1, p1);
				}
				str = i18n("MainWin.unexpectedUserID");
				str = CDcreator.replaceAll(str, "@USER-ID@", userID);
				if (showConfirmDialog(str,
						i18n("warning"), JOptionPane.YES_NO_OPTION) == 0)
					writeHint(no + "setAcceptKey2 " + userID);
				break;
			default:
				;
		}
	}

	void processHint(String str) {
//		System.out.println("processHint " + str);
		try {
			if (str.startsWith("rank"))
				parseRankHint(str);
			else
			if (str.startsWith("Track"))
				parseCDrecordLine(str);
			else
			if (str.startsWith("capacity"))
				parseCapacityHint(str);
			else
			if (str.startsWith(hintPercentDone))
				parsePercentDoneHint(str);
		}
		catch (IOException e) {
		}

		if (str.equals(hintWriterReserved)) {
			JPanel vbox = new JPanel();
			vbox.setLayout(new BoxLayout(vbox, BoxLayout.Y_AXIS));
			vbox.add(Box.createVerticalStrut(10));

			JComponent viewer = MainWin.createViewer(infoStr);
			viewer.setAlignmentX(JPanel.LEFT_ALIGNMENT);
			vbox.add(viewer);
			vbox.add(Box.createVerticalStrut(5));

			Object[] options = { cancelButton, offlineButton };
			optionPane = new JOptionPane(
				vbox,
				JOptionPane.INFORMATION_MESSAGE,
				JOptionPane.DEFAULT_OPTION,
				null, options);
			show(optionPane, i18n("titleInsertCD"));
			infoStr = "";
		}
		else
		if (str.equals(hintCDdetected)) {
			mode = MEDIUM_DETECTED;
			Log.put("Medium detected");
			burnDialog.setRange(0, 100);
			show(burnDialog, i18n("titleBurnSessionControl"));
		}
		else
		if (str.equals(hintBlankingTOC)) {
			Log.put("");
			Log.put("cdrecord blank=fast started ==>");
			burnDialog.setMessage(i18n("blankingTOC"));
		}
		else
		if (str.equals(hintDVDFormatStarted)) {
			Log.put("");
			Log.put("dvd+rw-format started ==>");
			burnDialog.setMessage(i18n("BurnDialog.formatting"));
		}
		else
		if (str.equals(hintMakingImage)) {
			Log.put("");
			Log.put("mkisofs started ==>");
			setTitle(i18n("titleCreateImage"));
			burnDialog.setMessage(i18n("createImage"));
		}
		else
		if (str.equals(hintCDrecordStarted)) {
			Log.put("");
			Log.put("cdrecord started ==>");
			setTitle(i18n("titleBurn"));
			burnDialog.setMessage(i18n("burn"));
			burnDialog.setProgress(0);
//			showDialog(burnDialog);
		}
		else
		if (str.equals(hintGrowisofsStarted)) {
			Log.put("");
			Log.put("growisofs started ==>");
			setTitle(i18n("titleBurn"));
			burnDialog.setMessage(i18n("burn"));
			burnDialog.setProgress(0);
		}
		else
		if (str.equals(hintFixating)) {
			burnDialog.setMessage(i18n("fixating"));
			burnDialog.setProgress(0);
//			burnDialog.setIndeterminate(true);
			burnDialog.setRange(0, 100);
		}
		else
		if (str.equals(hintCDwasClosed)) {
			CDwasClosed = true;
		}
		else
		if (str.equals(hintVerifying)) {
			mode = VERIFY;
			Log.put("");
			Log.put("CDWverify started ==>");
			setTitle(i18n("titleVerifying"));
			burnDialog.setMessage(i18n("verifying"));
			burnDialog.setProgress(0);
			burnDialog.setRange(0, 100);
			burnDialog.addButton(cancelButton);
		}
		else
		if (str.equals("reload medium")) {
			burnDialog.setMessage(i18n("BurnDialog.reload"));
			burnDialog.setIndeterminate(true);
		}
		else
		if (str.equals("reload medium done")) {
			burnDialog.setMessage(i18n("verifying"));
			burnDialog.setIndeterminate(false);
			burnDialog.setProgress(0);
			burnDialog.setRange(0, 100);
		}
		else
		if (str.startsWith("Medium ")
				|| str.indexOf("-> eject") >= 0) {
			Log.put(str);
			final Object panel = optionPane.getMessage();
			if (panel instanceof JPanel
					&& Log.getView().getParent() != panel)
				try {
					SwingUtilities.invokeAndWait(new Runnable() {
						public void run() {
//							Log.getView().setPreferredSize(
//								new Dimension(160, 100));
							Log.getView().setAlignmentX(
								JPanel.LEFT_ALIGNMENT);
							((JPanel)panel).add(Log.getView());
//							activeDialog.pack();
/*							Dimension d = activeDialog.getSize();
							d.height += 100;
							activeDialog.setSize(d);
							CDcreator.view.center(activeDialog);
*/
							show(optionPane, i18n("titleInsertCD"));
						}
					});
				}
				catch (Exception e) {
				}
		}
	}

	void processInfo(String str) {
		String masterCD = "Please insert the master CD into the drive No.";
		if (str.indexOf(masterCD) >= 0) {
			str = CDcreator.replaceAll(str,
				masterCD, i18n("Burn.insertMaster"));
			str += "<br><br>\n";
		}
		super.processInfo(str);
	}

	void processLog(String str) {
		if (str.startsWith("Track ")) {
			int p0 = str.indexOf("[buf ");
			if (p0 > 0) {
				int p1 = str.indexOf("%]", p0);
				if (p1 > p0) {
					p0 += 4;
					while (str.charAt(p0) == ' ')
						p0++;
					String str2 = str.substring(p0, p1);
					try {
						int n = Integer.parseInt(str2);
						burnDialog.setBuffer(n);
					}
					catch (Exception e) {
					}
				}
			}
		}
	}

	public void run() {
		Object[] options = { cancelButton, offlineButton };
		optionPane = new JOptionPane(
			i18n("reserve"),
			JOptionPane.INFORMATION_MESSAGE,
			JOptionPane.DEFAULT_OPTION,
			MainWin.getIcon("burn.png"), options, options[1]);
		showCentered(optionPane, i18n("titleReserve"));

		execCommand("resetOptions");
		execOptions();

		if (Options.format)
			execCommand("setFormat");

		if (Options.speed > 0)
			execCommand("setSpeed " + Options.speed);

		mode = WAITING_FOR_WRITER;
		execBurnCommand();
		burnSessionResult = lastResult;
		burnSessionResultOK = lastResultOK();

		super.run();
	}

	void showResult() {
		if (cancel && mode != VERIFY)
			return;

		JDialog dialog;
		if (burnSessionResultOK)
			dialog = createSuccessDialog();
		else
			dialog = createErrorDialog();

		while (true) {
			optionPane.setInitialValue(options[1]);
			optionPane.setValue(
				JOptionPane.UNINITIALIZED_VALUE);
			dialog.show();
			if (optionPane.getValue() == options[0])
				new LogWin().show();
			else
				break;
		}
	}
}
