/*
 * @(#)BurnDialog.java
 *
 * This file is part of webCDwriter - Network CD/DVD Writing.
 *
 * Copyright (C) 1999-2005 Jrg P. M. Haeger
 *
 * webCDwriter is free software. See CDcreator.java for details.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * BurnDialog
 *
 * @version 20050830
 * @author Jrg P. M. Haeger
 */
class BurnDialog extends javax.swing.JPanel {
	Command parent;
	JLabel label;
	JProgressBar bar, bufferBar = null;
	JPanel buttons;
	JPanel hideBar;
	JPanel showBar;

	BurnDialog(Command aParent) {
		parent = aParent;
		setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		label = new JLabel();
		label.setAlignmentX(CENTER_ALIGNMENT);

		bar = new JProgressBar();
		bar.setStringPainted(true);

		JButton button = new JButton(i18n("details"));
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				prepareDetails();
				removeAll();
				addMain();
				add(hideBar);
				add(Log.getView());
				parent.validate();
			}
		});
		buttons = new JPanel();
		buttons.setLayout(new FlowLayout());
		buttons.add(button);

		add(Box.createVerticalGlue());
		addMain();
		add(Box.createVerticalStrut(15));
		add(buttons);
	}

	void addButton(JButton button) {
		buttons.add(button);
		parent.validate();
	}

	private void addMain() {
		add(label);
		add(Box.createVerticalStrut(10));
		add(bar);
		if (bufferBar != null) {
			add(Box.createVerticalStrut(10));
			add(bufferBar);
		}
	}

	static String i18n(String key) {
		if (key.startsWith("."))
			key = "BurnDialog" + key;
		return CDcreator.i18n(key);
	}

	void prepareDetails() {
		JButton hide = new JButton(MainWin.getIcon("up.png"));
		hide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeAll();
				addMain();
				add(showBar);
				parent.pack();
			}
		});
		hide.setPreferredSize(new Dimension(25, 25));
		hideBar = new JPanel();
		hideBar.setLayout(
			new BoxLayout(hideBar, BoxLayout.X_AXIS));
		hideBar.add(Box.createHorizontalGlue());
		hideBar.add(hide);

		Log.getView().setAlignmentX(JPanel.CENTER_ALIGNMENT);
		Log.getView().setPreferredSize(new Dimension(400, 160));

		JButton show = new JButton(MainWin.getIcon("down.png"));
		show.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeAll();
				addMain();
				add(hideBar);
				add(Log.getView());
				parent.pack();
			}
		});
		show.setPreferredSize(new Dimension(25, 25));
		showBar = new JPanel();
		showBar.setLayout(
			new BoxLayout(showBar, BoxLayout.X_AXIS));
		showBar.add(Box.createHorizontalGlue());
		showBar.add(show);
	}

	public void setBuffer(final int percent) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				if (bufferBar == null) {
					bufferBar = new JProgressBar(0, 100);
					bufferBar.setString(i18n(".buffer"));
					bufferBar.setStringPainted(true);

					removeAll();
					add(Box.createVerticalGlue());
					addMain();
					add(Box.createVerticalStrut(15));
					add(buttons);
					parent.validate();
				}
				bufferBar.setValue(percent);
			}
		});
	}

	public void setIndeterminate(final boolean newValue) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				bar.setIndeterminate(newValue);
				bar.setStringPainted(!newValue);
			}
		});
	}

	public void setMessage(final String str) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				label.setText(str);
				if (bufferBar != null) {
					remove(bufferBar);
					bufferBar = null;
					parent.validate();
				}
			}
		});
	}

	public void setRange(int a, int b) {
		bar.setMinimum(a);
		bar.setMaximum(b);
	}

	public void setProgress(final int n) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				bar.setValue(n);
				if (bufferBar != null) {
					if (n == bar.getMaximum())
						bufferBar.setValue(0);
				}
			}
		});
	}
}
